import {
  ConflictException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import {
  ChildPaginationDto,
  CreateByUserChildDto,
  CreateUserChildDto,
} from './dto/create-user-child.dto';
import { UpdateUserChildDto } from './dto/update-user-child.dto';
import { InjectRepository } from '@nestjs/typeorm';
import { UserChild } from './entities/user-child.entity';
import { Repository } from 'typeorm';
import { UserDetail } from 'src/user-details/entities/user-detail.entity';
import { DefaultStatus, MemberStatus, UserRole } from 'src/enum';
import { MembershipCard } from 'src/membership-card/entities/membership-card.entity';
import { generateQrCode } from 'src/utils/qrCode.util';

@Injectable()
export class UserChildService {
  constructor(
    @InjectRepository(UserChild) private readonly repo: Repository<UserChild>,
    @InjectRepository(MembershipCard)
    private readonly memCardRepo: Repository<MembershipCard>,
    @InjectRepository(UserDetail)
    private readonly udRepo: Repository<UserDetail>,
  ) {}

  async create(dto: CreateUserChildDto) {
    const user = await this.udRepo.findOne({
      where: { accountId: dto.accountId },
    });
    if (!user) {
      throw new NotFoundException('User Not Found!');
    }
    const card = await this.memCardRepo.findOne({
      where: { id: user.membershipCardId },
    });
    if (!card) {
      throw new NotFoundException('MembershipCard Not found');
    }
    const memberCount = card.memberCount;
    const childCount = await this.repo.count({
      where: { accountId: dto.accountId },
    });
    if (childCount == memberCount) {
      throw new ConflictException('Cannot add child member, limit exceeded!');
    }
    if (user.status != MemberStatus.ACTIVE) {
      throw new ConflictException('Member is not acitve');
    }
    const child = await this.repo.findOne({
      where: { accountId: dto.accountId, name: dto.name },
    });
    if (child) {
      throw new ConflictException(
        'Child Member already exists with this name!',
      );
    }
    const obj = Object.assign(dto);
    return this.repo.save(obj);
  }

  async addChild(dto: CreateByUserChildDto) {
    const user = await this.udRepo.findOne({
      where: { accountId: dto.accountId },
    });
    if (!user) {
      throw new NotFoundException('User Not Found!');
    }
    const card = await this.memCardRepo.findOne({
      where: { id: user.membershipCardId },
    });
    if (!card) {
      throw new NotFoundException('MembershipCard Not found');
    }
    const memberCount = card.memberCount;
    const childCount = await this.repo.count({
      where: { accountId: dto.accountId },
    });
    if (childCount == memberCount) {
      throw new ConflictException('Cannot add child member, limit exceeded!');
    }
    if (user.status != MemberStatus.ACTIVE) {
      throw new ConflictException('Member is not acitve');
    }
    const child = await this.repo.findOne({
      where: { accountId: dto.accountId, name: dto.name },
    });
    if (child) {
      throw new ConflictException(
        'Child Member already exists with this name!',
      );
    }
    dto.businessAccId = user.createdById;
    const obj = Object.assign(dto);
    return this.repo.save(obj);
  }

  async findOne(id: string) {
    return this.repo.findOne({ where: { id } });
  }

  async getChildQrCode(userChildId: string) {
    const userProfile = await this.repo
      .createQueryBuilder('userChild')
      .leftJoinAndSelect('userChild.account', 'account')
      .select([
        'userChild.id',
        'userChild.memberId',

        'account.id',
        'account.phoneNumber',
        'account.roles',
        'account.createdAt',
      ])
      .where('userChild.id = :id', { id: userChildId })
      .getOne();
    if (!userProfile) {
      throw new Error('User not found');
    }
    // return userProfile;
    const qrCode = await generateQrCode(userProfile);
    return { qrCode: qrCode };
  }

  async find(dto: ChildPaginationDto, businessAccId: string) {
    const keyword = dto.keyword || '';
    const startDate = new Date(dto.startDate);
    startDate.setHours(0, 0, 0, 0);
    const endDate = new Date(dto.endDate);
    endDate.setHours(23, 59, 59, 999);

    const query = await this.repo
      .createQueryBuilder('userChild')
      .leftJoinAndSelect('userChild.account', 'account')
      .leftJoinAndSelect('account.userDetail', 'userDetail')
      .leftJoinAndSelect('userDetail.membershipCard', 'membershipCard')
      .select([
        'userChild.id',
        'userChild.businessAccId',
        'userChild.memberId',
        'userChild.name',
        'userChild.email',
        'userChild.phoneNumber',
        'userChild.relation',
        'userChild.martialStatus',
        'userChild.dob',
        'userChild.profile',
        'userChild.createAt',

        'account.id',
        'account.phoneNumber',
        'account.roles',
        'account.createdAt',

        'userDetail.id',
        'userDetail.memberId',
        'userDetail.fName',
        'userDetail.mName',
        'userDetail.lName',
        'userDetail.email',
        'userDetail.gender',
        'userDetail.profile',
        'userDetail.membershipValidFrom',
        'userDetail.membershipValidTo',
        'userDetail.cardNumber',
        'userDetail.salutation',
        'userDetail.haveBusiness',
        'userDetail.status',
        'userDetail.paymentStatus',

        'membershipCard.id',
        'membershipCard.name',
        'membershipCard.validity',
        'membershipCard.price',
        'membershipCard.currencyType',
        'membershipCard.memberCount',
      ])
      .where('userChild.businessAccId = :businessAccId', {
        businessAccId: businessAccId,
      });
    if (dto.status && dto.status.length > 0) {
      query.andWhere('userDetail.status = :status', {
        status: dto.status,
      });
    }
    if (dto.phoneNumber && dto.phoneNumber.length > 0) {
      query.andWhere('userChild.phoneNumber = :phoneNumber', {
        phoneNumber: dto.phoneNumber,
      });
    }
    if (dto.membershipType && dto.membershipType.length > 0) {
      query.andWhere('membershipCard.name = :name', {
        name: dto.membershipType,
      });
    }
    if (dto.memberId && dto.memberId.length > 0) {
      query.andWhere('userChild.memberId = :memberId', {
        memberId: dto.memberId,
      });
    }
    if (dto.startDate && dto.endDate) {
      query.andWhere(
        'userDetail.membershipValidTo >= :startDate AND userDetail.membershipValidTo <= :endDate',
        {
          startDate: startDate,
          endDate: endDate,
        },
      );
    }
    const [result, total] = await query
      .orderBy({ 'userChild.createAt': 'DESC' })
      .take(dto.limit)
      .skip(dto.offset)
      .getManyAndCount();

    return { result, total };
  }

  async findChild(id: string) {
    const result = await this.repo
      .createQueryBuilder('userChild')
      .leftJoinAndSelect('userChild.account', 'account')
      .leftJoinAndSelect('account.userDetail', 'userDetail')
      .leftJoinAndSelect('userDetail.membershipCard', 'membershipCard')
      .leftJoinAndSelect('membershipCard.cardAmenities', 'cardAmenities')
      .select([
        'userChild.id',
        'userChild.memberId',
        'userChild.businessAccId',
        'userChild.name',
        'userChild.email',
        'userChild.phoneNumber',
        'userChild.relation',
        'userChild.martialStatus',
        'userChild.dob',
        'userChild.profile',
        'userChild.createAt',

        'account.id',
        'account.phoneNumber',
        'account.roles',
        'account.createdAt',

        'userDetail.id',
        'userDetail.memberId',
        'userDetail.membershipValidFrom',
        'userDetail.membershipValidTo',
        'userDetail.fName',
        'userDetail.mName',
        'userDetail.lName',
        'userDetail.email',
        'userDetail.gender',
        'userDetail.profile',
        'userDetail.address1',
        'userDetail.address2',
        'userDetail.city',
        'userDetail.state',
        'userDetail.zipcode',
        'userDetail.memberDoc',
        'userDetail.businessType',
        'userDetail.businessName',
        'userDetail.gstNumber',
        'userDetail.businessDoc',
        'userDetail.businessCity',
        'userDetail.businessState',
        'userDetail.businessZipcode',
        'userDetail.businessPhone',
        'userDetail.cardNumber',
        'userDetail.landMark',
        'userDetail.fatherName',
        'userDetail.dob',
        'userDetail.qualification',
        'userDetail.profession',
        'userDetail.panNumber',
        'userDetail.income',
        'userDetail.salutation',
        'userDetail.haveBusiness',
        'userDetail.businessAddress1',
        'userDetail.businessAddress2',
        'userDetail.pan',
        'userDetail.aadhar',
        'userDetail.aadharNumber',
        'userDetail.status',
        'userDetail.paymentStatus',

        'membershipCard.id',
        'membershipCard.name',
        'membershipCard.validity',
        'membershipCard.price',
        'membershipCard.currencyType',
        'membershipCard.memberCount',

        'cardAmenities.id',
        'cardAmenities.name',
        'cardAmenities.icon',
        'cardAmenities.desc',
        'cardAmenities.shortDesc',
      ])
      .where('userChild.id = :id', { id: id })
      .getOne();

    return result;
  }

  async update(id: string, dto: UpdateUserChildDto) {
    const result = await this.repo.findOne({ where: { id } });
    if (!result) {
      throw new NotFoundException('Child Not Found!');
    }
    const obj = Object.assign(result, dto);
    return this.repo.save(obj);
  }

  async profileImage(image: string, result: UserChild) {
    const obj = Object.assign(result, {
      profile: process.env.PV_CDN_LINK + image,
      profilePath: image,
    });
    return this.repo.save(obj);
  }

  async profileImageByUser(image: string, result: UserChild) {
    const obj = Object.assign(result, {
      profile: process.env.PV_CDN_LINK + image,
      profilePath: image,
    });
    return this.repo.save(obj);
  }

  async docByUser(image: string, result: UserChild) {
    const obj = Object.assign(result, {
      doc: process.env.PV_CDN_LINK + image,
      docPath: image,
    });
    return this.repo.save(obj);
  }
}
