import {
  ConflictException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { CreateUserLogDto, LogPaginationDto } from './dto/create-user-log.dto';
import { UpdateUserLogDto } from './dto/update-user-log.dto';
import { InjectRepository } from '@nestjs/typeorm';
import { UserLog } from './entities/user-log.entity';
import { Brackets, Repository } from 'typeorm';
import { MemberEntranceType, MemberStatus } from 'src/enum';
import { UserDetail } from 'src/user-details/entities/user-detail.entity';

@Injectable()
export class UserLogService {
  constructor(
    @InjectRepository(UserLog) private readonly repo: Repository<UserLog>,
    @InjectRepository(UserDetail)
    private readonly udRepo: Repository<UserDetail>,
  ) {}

  async create(dto: CreateUserLogDto, accountId: string) {
    const user = await this.udRepo.findOne({ where: { id: dto.userDetailId } });
    if (!user) {
      throw new NotFoundException('Member Not Found!');
    }
    if (user.status != MemberStatus.ACTIVE) {
      throw new ConflictException('Member is not active!');
    }

    const date = new Date().toLocaleDateString('en-CA');
    const lastCheckIn = await this.repo.findOne({
      where: {
        userDetailId: dto.userDetailId,
        accountId: accountId,
        date: date,
      },
      order: { entryTime: 'DESC' },
    });
    if (lastCheckIn && lastCheckIn.exitTime == null) {
      throw new ConflictException('User must exit before checking in again!');
    }

    const currTime = new Date().toLocaleTimeString('en-IN', {
      timeZone: 'Asia/Kolkata',
      hour12: false,
    });
    const obj = Object.assign({
      accountId: accountId,
      userDetailId: dto.userDetailId,
      name: `${user.fName} ${user.mName} ${user.lName}`,
      date: date,
      entryTime: currTime,
      status: MemberEntranceType.ENTERED,
    });
    return this.repo.save(obj);
  }

  async findAll(dto: LogPaginationDto) {
    const keyword = dto.keyword || '';
    const query = await this.repo
      .createQueryBuilder('userLog')
      .where('userLog.userDetailId = :userDetailId', {
        userDetailId: dto.userDetailId,
      });
    if (dto.status.length > 0) {
      query.andWhere('userLog.status = :status', {
        status: dto.status,
      });
    }
    const [result, total] = await query
      .andWhere(
        new Brackets((qb) => {
          qb.where('userLog.name LIKE :keyword OR userLog.date LIKE :keyword', {
            keyword: '%' + keyword + '%',
          });
        }),
      )
      .orderBy({
        'userLog.date': 'DESC',
        'userLog.entryTime': 'DESC',
      })
      .take(dto.limit)
      .skip(dto.offset)
      .getManyAndCount();
    return { result, total };
  }

  async update(accountId: string, dto: UpdateUserLogDto) {
    const lastCheckIn = await this.repo.findOne({
      where: {
        userDetailId: dto.userDetailId,
        accountId: accountId,
        status: MemberEntranceType.ENTERED,
      },
      order: { entryTime: 'DESC' },
    });
    if (!lastCheckIn) {
      throw new NotFoundException('Member not entered yet!');
    }

    const currTime = new Date().toLocaleTimeString('en-IN', {
      timeZone: 'Asia/Kolkata',
      hour12: false,
    });

    const entryTime = lastCheckIn.entryTime;
    const parseTimeString = (timeStr: string): Date => {
      const [hours, minutes, seconds] = timeStr.split(':').map(Number);
      const now = new Date();
      now.setHours(hours, minutes, seconds, 0);
      return now;
    };

    const entryDate = parseTimeString(entryTime);
    const currDate = parseTimeString(currTime);

    // Calculate duration in milliseconds
    const durationMs = currDate.getTime() - entryDate.getTime();
    const durationInSeconds = Math.floor(durationMs / 1000);

    const obj = Object.assign(lastCheckIn, {
      exitTime: currTime,
      duration: durationInSeconds,
      status: MemberEntranceType.EXITED,
    });
    return this.repo.save(obj);
  }
}
